package agenda;

import java.util.*;
import java.io.*;
import java.lang.reflect.*;


public class LocalUserSession extends SessionManager
{
	
	private static final int loglimit = 10;
	
	private int logcount;
	
	private TimeTable agenda;
	private TTSimpleDumpManager sdm;
	
	public LocalUserSession(Agenda agenda)
	{
		this.agenda = agenda.getTT();
		this.sdm = agenda.getSDM();
		logcount = 0;
	}
	
	
	public Object getLock()
		{
			return this.agenda;			
		}

	
	public void open()
	{}
	
	public void close()throws InvalidClassException,IOException
	{
		sdm.dump();
		logcount = 0;	
	}
	
	public boolean add(Evento ev)throws InvalidClassException,NotSerializableException,IOException
		{
			synchronized(agenda)
			{
				if(agenda.add(ev))
						{
							Object[] arg = new Object[1];
							arg[0] = ev;
							Class[] par = new Class[1];
							par[0] = ev.getClass().getSuperclass();
							try
								{
									sdm.logaction((agenda.getClass()).getMethod("add",par),arg);
								}
								catch(NoSuchMethodException nsme)
										{
											System.err.println(nsme.toString());
											nsme.printStackTrace();
										}
							logcount++;
							if(logcount >= loglimit)
									{
										sdm.dump();
										logcount = 0;
									}
							return true;
						}
				else return false;
			}	
		}
	
	public boolean testforadd(Evento ev)
		{
			return agenda.testforadd(ev);
		}
	
	public boolean remove(Evento ev)throws InvalidClassException,NotSerializableException,IOException
		{
			synchronized(agenda)
			{
				if(agenda.remove(ev))
						{
							Object[] arg = new Object[1];
							arg[0] = ev;
							Class[] par = new Class[1];
							par[0] = ev.getClass().getSuperclass();
							try
								{
									sdm.logaction((agenda.getClass()).getMethod("remove",par),arg);
								}
								catch(NoSuchMethodException nsme)
										{
											System.err.println(nsme.toString());
											nsme.printStackTrace();
										}
							logcount++;
							if(logcount >= loglimit)
									{
										sdm.dump();
										logcount = 0;
									}
							return true;
						}
				else return false;
			}	

		}
	
	public void clear()throws InvalidClassException,NotSerializableException,IOException
		{
			synchronized(agenda)
			{
				agenda.clear();
				try
					{
						sdm.logaction((agenda.getClass()).getMethod("clear",null),null);
					}
					catch(NoSuchMethodException nsme)
										{
											System.err.println(nsme.toString());
											nsme.printStackTrace();
										}
				logcount++;
				if(logcount >= loglimit)
							{
								sdm.dump();
								logcount = 0;
							}
			}	
	
		}
	
	public boolean contains(Evento ev)
		{
			return agenda.contains(ev);
		}
	
	public boolean isEmpty()
		{
			return agenda.isEmpty();
		}
	
	public Iterator iterator()
		{
			return agenda.iterator();
		}
	
	public TimeTable testforadd(TimeTable ttb)
		{
			return agenda.testforadd(ttb);
		}
	
	public void sum(TimeTable ttb)throws InvalidClassException,NotSerializableException,IOException

		{
			synchronized(agenda)
			{
				agenda.sum(ttb);
				Object[] arg = new Object[1];
				arg[0] = ttb;
				Class[] par = new Class[1];
				par[0] = ttb.getClass();
				try
					{
						sdm.logaction((agenda.getClass()).getMethod("sum",par),arg);
					}
					catch(NoSuchMethodException nsme)
										{
											System.err.println(nsme.toString());
											nsme.printStackTrace();
										}
				logcount++;
				if(logcount >= loglimit)
						{
							sdm.dump();
							logcount = 0;
						}
			}	
		}
	
	public TimeTable difference(TimeRange tr)
		{
			return agenda.difference(tr);
		}
	
	public TimeTable difference(TimeTable ttb)
		{
			return agenda.difference(ttb);
		}
	
	public TimeTable intersection(TimeRange tr)
		{
			return agenda.intersection(tr);
		}
	
	public TimeTable intersection(TimeTable ttb)
		{
			return agenda.intersection(ttb);
		}


	
}//fine localuser