package agenda;

import java.io.*;
import java.lang.reflect.*;


public abstract class LogManager
{
	
	protected static class LogAction implements Serializable
	{
		private static final int mark = 0;
		/*intero positivo (basterebbe diverso da -1)
		 che separa i vari oggetti nello stream e
		 serve a trovare l'end of file
		 */
		
		
		private LogAction(String name,Class[] parameters,Object[] args)
			{
				this.name = name;
				this.parameters =	parameters;
				this.args = args;
			}
		
		private String name;
		private Class[] parameters;
		private Object[] args;
		
	
		private void writeObject(ObjectOutputStream stream) throws IOException
		{
			stream.writeByte(mark);
			stream.writeObject(name);
			stream.writeObject(parameters);
			stream.writeObject(args);
		}
 
 		private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException
		{
			if((stream.read())>=0)
					{
						name = ((String)stream.readObject());
						parameters = ((Class[])stream.readObject());
						args = ((Object[])stream.readObject());
					}
			else
					{
						name = "";
						parameters = null;
						args = null	;
					}
		}
 		 

		protected Object[] getArgs()
		{
			return this.args;
		}
		
		protected String getName()
		{
			return this.name;
		}

		protected Class[] getParameters()
		{
			return this.parameters;
		}
	
	
	}//fine classe LogAction

	
	
	protected String logfn;	//nome del file di log delle operazioni sulla timetable
	
	protected File log;//file di log: registra tutte le azioni eseguite sulla table dall'ultimo dump
		
	
	public synchronized void logaction(Method mtd,Object[] args) throws InvalidClassException,NotSerializableException,IOException
	{
		ObjectOutputStream logoos = new ObjectOutputStream(new FileOutputStream(logfn,true));
				
		logoos.writeObject(new LogAction(mtd.getName(),mtd.getParameterTypes(),args));
		logoos.flush();
		logoos.close();
	}
	
	/*operazione di dump : dovrebbe essere atomica,
		se il sistema cade durante questa operazione
		i file sono lasciati in uno stato inconsistente
		e il recovery  impossibile o pericoloso
	*/
		
	public abstract void dump() throws InvalidClassException,IOException;

	
	/*operazione di recovery : si suppone sia eseguita su una table nuova,
		a partire da un dump file e un log file
	*/
	
	public abstract void recovery() throws ClassNotFoundException,InvalidClassException,OptionalDataException,NotSerializableException,NoSuchMethodException,IllegalAccessException,IllegalArgumentException,InvocationTargetException,NullPointerException,ExceptionInInitializerError,StreamCorruptedException,IOException;
	
}//fine classe