package agenda;

import java.io.*;
import java.lang.reflect.*;


public class SimpleLogManager
{
	
	
	private class LogAction implements Serializable
	{
		private static final int mark = 0;
		/*intero positivo (basterebbe diverso da -1)
		 che separa i vari oggetti nello stream e
		 serve a trovare l'end of file
		 */
		
		
		private LogAction(String name,Class[] parameters,Object[] args)
			{
				this.name = name;
				this.parameters =	parameters;
				this.args = args;
			}
		
		private String name;
		private Class[] parameters;
		private Object[] args;
		
	
		private void writeObject(ObjectOutputStream stream) throws IOException
		{
			stream.writeByte(mark);
			stream.writeObject(name);
			stream.writeObject(parameters);
			stream.writeObject(args);
		}
 
 		private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException
		{
			if((stream.read())>=0)
					{
						name = ((String)stream.readObject());
						parameters = ((Class[])stream.readObject());
						args = ((Object[])stream.readObject());
					}
			else
					{
						name = "";
						parameters = null;
						args = null	;
					}
		}
 		 
	}//fine classe LogAction

	
	
	private final TimeTable table;//la timetable che devo gestire
	
	private String logfn;	//nome del file di log delle operazioni sulla timetable
	private String dumpfn;
	
	
	private File log;//file di log: registra tutte le azioni eseguite sulla table dall'ultimo dump
	private File dump;
	
	
	public SimpleLogManager(TimeTable table,File group) throws ClassNotFoundException,IOException
	{
		this.table = table;
		
		ObjectInputStream groupois = new ObjectInputStream(new FileInputStream(group));
		
		log = ((File)groupois.readObject());
		dump = ((File)groupois.readObject());
		
		groupois.close();

		logfn = log.getPath();
		dumpfn = dump.getPath();
		
		log.createNewFile();		
		dump.createNewFile();
	
	}
	
	
	public synchronized void logaction(Method mtd,Object[] args) throws InvalidClassException,NotSerializableException,IOException
	{
		ObjectOutputStream logoos = new ObjectOutputStream(new FileOutputStream(logfn,true));
				
		logoos.writeObject(new LogAction(mtd.getName(),mtd.getParameterTypes(),args));
		logoos.flush();
		logoos.close();
	}
	
	/*operazione di dump : dovrebbe essere atomica,
		se il sistema cade durante questa operazione
		i file sono lasciati in uno stato inconsistente
		e il recovery  impossibile o pericoloso
	*/
		
	public synchronized void dump() throws InvalidClassException,IOException
	{
		synchronized(table)
			{
				ObjectOutputStream dumpoos = new ObjectOutputStream(new FileOutputStream(dump));
				try
					{
						dumpoos.writeObject(table);
					}
					catch(NotSerializableException nse)	// impossibile che si verifichi questa eccezione, TimeTable  serializable
						{
							System.err.println(nse.toString());
						}
				dumpoos.flush();
				dumpoos.close();
		
				FileOutputStream logfos = new FileOutputStream(log);
				logfos.close();
			}		
	}

	
	/*operazione di recovery : si suppone sia eseguita su una table nuova,
		a partire da un dump file e un log file
	*/
	
	public synchronized void recovery() throws ClassNotFoundException,InvalidClassException,OptionalDataException,NotSerializableException,NoSuchMethodException,IllegalAccessException,IllegalArgumentException,InvocationTargetException,NullPointerException,ExceptionInInitializerError,StreamCorruptedException,IOException
	{
		synchronized(table)
			{				
				ObjectInputStream dumpois = new ObjectInputStream(new FileInputStream(dump));
		
		
				TimeTable ttb = ((TimeTable)dumpois.readObject());
				table.sum(ttb);
				dumpois.close();
			
				ObjectInputStream logois = new ObjectInputStream(new FileInputStream(log));
		
				LogAction temp = ((LogAction)logois.readObject());
	
				while((temp.name)!="")
					{
						((table.getClass()).getMethod(temp.name,temp.parameters)).invoke(table,temp.args);
						temp = ((LogAction)logois.readObject());
					}
		
				logois.close();
			}
	}

	
	
}//fine classe