package agenda;

import SOMA.*;
import SOMA.agent.*;
import SOMA.agent.AgentWorker.*;
import SOMA.agent.mobility.*;
import SOMA.naming.*;

import java.io.*;
import java.util.*;



/*Questa classe descrive lo stato del sistema, cio
	raccoglie tutte le info utili per il recovery
	*/


public class SystemStatus implements Serializable
{
	private static boolean valid = false;
	
	private static File statusFile;// il file su cui serializzare se stesso
	
	private static SystemStatus singleton;//la variabile singleton
	
		
	
	public static boolean initStatus(File file) throws ClassNotFoundException,IOException
	{
		if((file != null)&&(!(valid)))
		{
			statusFile = file;
			try
				{
					if(statusFile.exists())
						{//il file esiste gi, allora lo leggo
							ObjectInputStream ois = new ObjectInputStream(new FileInputStream(statusFile));
							singleton = ((SystemStatus)ois.readObject());
							ois.close();				
						}
					else
						{
							//situazione nuova quindi normale
							singleton = new SystemStatus();
						}	
					valid = true;					
					return valid;
				}
				catch(ClassNotFoundException cnfe)
						{
							statusFile = null;
							singleton = null;
							valid = false;
							System.err.println(cnfe);
							throw cnfe;
						}					
				catch(IOException ioe)
						{
							statusFile = null;
							singleton = null;
							valid = false;
							System.err.println(ioe);
							throw ioe;
						}					
			}
			else return false;//non mi dovevi chiamare		
		
	}//fine init
	
	
	
	public static SystemStatus getInstance()
	{
		if(valid)return singleton;
		else return null;
	}//fine getInstance
			
	
	public static boolean dump()throws IOException
	{
		if(valid)
				{
					ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream(statusFile));
					oos.writeObject(singleton);
					oos.close();
					return true;
				}
		else return false;

	}//fine dump
	
	
	
	
	
	/* i file attualmente aperti dal sistema 
		per caricare l'agenda e la rubrica
	*/
		
	private File agendaFile;
	
	private File rubricaFile;
	
	
	/*un file gestito in append dove tutti gli agenti coordinatori 
	possono scrivere i loro reports
	*/
	
	public final File reportFile = new File("report.rpt");
	
	
	/* elenco degli agenti coordinatori attualmente attivi su questo place
	e che devono essere ripristinati in caso di crash
	*/
	
	private HashSet agentiCoord = new HashSet();

	
	
	/* elenco degli agenti di sessione attualmente attivi su questo place
	e che devono essere ripristinati in caso di crash: 
	il ripristino prevede anche la spedizione in mailbox di un SessionManager
	*/
	
	private HashSet agentiSessione = new HashSet();
	
	
	
	private SystemStatus()
	{}
	
	/*
	public void dump()throws IOException
	{
		SystemStatus.dump();
	}//fine dump
	*/
	
	public boolean setSessionReq(AgentID id)throws IOException
			{
				boolean result = agentiSessione.add(id);
				SystemStatus.dump();
				return result;
			}//fine setRestoreReq
			
	
	public boolean resetSessionReq(AgentID id)throws IOException
		{
			boolean result = agentiSessione.remove(id);
			SystemStatus.dump();
			return result;
		}//fine resetRestoreReq
	
	
	
	
	
	/*
	Metodi chiamati dagli agenti coordinatori
	*/
	
		public boolean setRestoreReq(AgentID id)throws IOException
			{
				boolean result = agentiCoord.add(id);
				SystemStatus.dump();
				return result;
			}//fine setRestoreReq
			
	
		public boolean resetRestoreReq(AgentID id)throws IOException
		{
			boolean result = agentiCoord.remove(id);
			SystemStatus.dump();
			return result;
		}//fine resetRestoreReq
	
	
		public void report(SessionReport rep)throws IOException
		{
			ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream(reportFile.getPath(),true));
			oos.writeObject(rep);
			oos.close();
		}//fine report


	public void loadAgents(Agenda agenda,Environment env)
	{
		try
			{
				env.agentManager.load();
			}
			catch(IOException ioe)
				{
					env.err.println(ioe);
					ioe.printStackTrace();
				}
			catch(ClassNotFoundException cnfe)
				{
					env.err.println(cnfe);
					cnfe.printStackTrace();
				}
		
		loadCoords(env);
		loadSessions(agenda,env);
		
	}//fine loadAgents
	
	public void loadCoords(Environment env)
	{
		Iterator cit = agentiCoord.iterator();
		
		while(cit.hasNext())
		{
			AgentID id = ((AgentID)cit.next());
			
			try
				{
					loadCoord(env,"agent_" + id.toString()+".sav");
				}
				catch(IOException ioe)
					{
						env.err.println(ioe);
						ioe.printStackTrace();
					}
				catch(ClassNotFoundException cnfe)
					{
						env.err.println(cnfe);
						cnfe.printStackTrace();
					}
		}//fine while
			
	}//fine loadCoord
		
	public void loadSessions(Agenda agenda,Environment env)
	{
		Iterator sit = agentiSessione.iterator();
		
		while(sit.hasNext())
		{
			AgentID id = ((AgentID)sit.next());
			
			try
				{
					loadSession(agenda,env,"agent_" + id.toString()+".sav");
				}
				catch(IOException ioe)
					{
						env.err.println(ioe);
						ioe.printStackTrace();
					}
				catch(ClassNotFoundException cnfe)
					{
						env.err.println(cnfe);
						cnfe.printStackTrace();
					}

		}//fine while
				
	}//fine loadSession
			
	public void loadCoord(Environment env,String fileName)throws IOException,ClassNotFoundException
	{
		ObjectInputStream ois = new ObjectInputStream(new FileInputStream(new File(fileName)));
		
		SavedWorker saved = ((SavedWorker)ois.readObject());
		ois.close();
		
		try
			{
				AgentID id = saved.restoreWorker(env);
				if(id != null)
					{
						AgentWorker aw = env.agentManager.agentWorkerStore.getWorker(id);
						if((aw != null)&&(aw.getStatus() != AgentWorker.ON))
							aw.start();
					};
			}
			catch(AgentWorkerException awe)
				{
					env.err.println(awe);
					awe.printStackTrace();
				}
	}//fine loadCoord
		
		
	public void loadSession(Agenda agenda,Environment env,String fileName)throws IOException,ClassNotFoundException
	{
		ObjectInputStream ois = new ObjectInputStream(new FileInputStream(new File(fileName)));
		
		SavedWorker saved = ((SavedWorker)ois.readObject());
		ois.close();
		
		try
			{
				AgentID id = saved.restoreWorker(env);
				if(id != null)
					{
						env.agentManager.sendMessage(new Message(new RemoteUserSession(agenda),id,id));
					};
			}
			catch(AgentWorkerException awe)
				{
					env.err.println(awe);
					awe.printStackTrace();
				}
	}//fine loadSession


	
	
	
	
	
}//fine classe