package agenda;

import java.io.*;
import java.lang.reflect.*;


public class TTSimpleDumpManager extends SimpleDumpManager
{
	
	private final TimeTable table;//la timetable che devo gestire
	
	
	public TTSimpleDumpManager(TimeTable table,File group) throws ClassNotFoundException,IOException
	{
		super(group);
		this.table = table;		
	
	}//fine costruttore
	
	
	
	/*operazione di dump : dovrebbe essere atomica,
		se il sistema cade durante questa operazione
		i file sono lasciati in uno stato inconsistente
		e il recovery  impossibile o pericoloso
	*/
		
	public synchronized void dump() throws InvalidClassException,IOException
	{
		synchronized(table)
			{
				ObjectOutputStream dumpoos = new ObjectOutputStream(new FileOutputStream(dump));
				try
					{
						dumpoos.writeObject(table);
					}
					catch(NotSerializableException nse)	// impossibile che si verifichi questa eccezione, TimeTable  serializable
						{
							System.err.println(nse.toString());
						}
				dumpoos.flush();
				dumpoos.close();
		
				FileOutputStream logfos = new FileOutputStream(log);
				logfos.close();
			}		
	}//fine dump

	
	/*operazione di recovery : si suppone sia eseguita su una table nuova,
		a partire da un dump file e un log file
	*/
	
	public synchronized void recovery() throws ClassNotFoundException,InvalidClassException,OptionalDataException,NotSerializableException,NoSuchMethodException,IllegalAccessException,IllegalArgumentException,InvocationTargetException,NullPointerException,ExceptionInInitializerError,StreamCorruptedException,IOException
	{
		synchronized(table)
			{				
				if(dump.exists())
					{
						ObjectInputStream dumpois = new ObjectInputStream(new FileInputStream(dump));
	
						TimeTable ttb = ((TimeTable)dumpois.readObject());
						table.sum(ttb);
						dumpois.close();
					}
			
				if(log.exists())
					{
						ObjectInputStream logois = new ObjectInputStream(new FileInputStream(log));
		
						LogAction temp = ((LogAction)logois.readObject());
	
						while((temp.getName())!="")
							{
								((table.getClass()).getMethod(temp.getName(),temp.getParameters())).invoke(table,temp.getArgs());
								temp = ((LogAction)logois.readObject());
							}
		
						logois.close();
					}//fine if
			}
	}//fine recovery

	
	
}//fine classe